class Day
{
public:
    /**
     * Public constructor.
     * Construct a day with a given year, month, and day
     * of the Julian/Gregorian calendar. The Julian calendar
     * is used for all days before October 15, 1582.
     */
    Day(int y, int m, int d)
        : year(y), month(m), date(d), julian(0),
          ymd_valid(true), julian_valid(false) {}

    /**
     * Getter.
     * @return the year of this day.
     */
    int get_year();

    /**
     * Getter.
     * @return the month of this day.
     */
    int get_month();

    /**
     * Getter.
     * @return the date of this day.
     */
    int get_date();

    /**
     * Return the day that is a given number of days away from this day.
     * @param n the number of days, can be negative.
     * @return the day.
     */
    Day *add_days(int n);

    /**
     * Return the number of days between this day and another day.
     * @param other the other day.
     * @return the number of days (> 0 if this day comes before)
     */
    int days_from(Day *other);

private:
    int year;
    int month;
    int date;
    int julian;

    bool ymd_valid;
    bool julian_valid;

    /**
     * Private constructor.
     * Construct a day with a given Julian date.
     * @param j the Julian date.
     */
    Day(const int j)
        : year(0), month(0), date(0), julian(j),
          ymd_valid(false), julian_valid(true) {}

    /**
     * Compute the Julian day number of this day if necessary.
     */
    void ensure_julian();

    /**
     * Convert this Julian day number to a calendar date if necessary.
     */
    void ensure_ymd();

    /**
     * Compute the Julian day number of a given day.
     * Positive year for CE, negative for BCE. The year after 1 BCE is 1 CE.
     * Algorithm from Press et al., Numerical Recipes in C, 2nd ed.,
     * Cambridge University Press, 1992.
     * @param year the year.
     * @param month the month.
     * @param date the day of the month.
     */
    static int to_julian(const int year, const int month, const int date);

    /**
     * Convert a Julian day number to a calendar date.
     * Algorithm from Press et al., Numerical Recipes in C, 2nd ed.,
     * Cambridge University Press, 1992.
     * @param j the Julian day number.
     * @param ymd the result array, [0] = year, [1] = month, [2] = day of month.
     */
    static void from_julian(const int j, int ymd[]);
};





